﻿using Microsoft.Extensions.Logging;
using Plugin.BLE;
using Plugin.BLE.Abstractions.Contracts;
using RobobinApp.Networking; 

namespace RobobinApp
{
    public partial class App : Application
    {
        public static IBluetoothLE BluetoothLE { get; private set; }
        public static IAdapter BluetoothAdapter { get; private set; }
        public static WifiManager WifiManager { get; private set; }

        public App()
        {
            InitializeComponent();
            ConfigureLogging();

     
            InitializeBluetoothAdapter();

            WifiManager = new WifiManager();
            Task.Run(() => WifiManager.StartListening());

            MainPage = new AppShell();
        }

        protected override Window CreateWindow(IActivationState activationState)
        {
            var window = base.CreateWindow(activationState);

            window.MinimumWidth = 800;
            window.MinimumHeight = 650;

            return window;
        }

        private void ConfigureLogging()
        {
            var loggerFactory = LoggerFactory.Create(builder =>
            {
                builder
                    .AddConsole()
                    .AddDebug();
            });

            Logger = loggerFactory.CreateLogger<App>();
        }

        private void InitializeBluetoothAdapter()
        {
            BluetoothLE = CrossBluetoothLE.Current;
            BluetoothAdapter = BluetoothLE.Adapter;
        }

        public ILogger<App> Logger { get; private set; }
    }
}
