﻿namespace Robobin.Models
{
    public class Node
    {
        public string Name { get; set; }
        public float X { get; set; }
        public float Y { get; set; }
        public List<Edge> Edges { get; set; } // Add edges to keep track of connections

        public Node(string name, float x, float y)
        {
            Name = name;
            X = x;
            Y = y;
            Edges = new List<Edge>(); // Initialize the edges list
        }
    }

    public class Edge
    {
        public Node TargetNode { get; set; }
        public float Cost { get; set; } // Cost to traverse this edge

        public Edge(Node targetNode, float cost)
        {
            TargetNode = targetNode;
            Cost = cost;
        }
    }

    public class Graph
    {
        public string Name { get; set; }
        public List<Node> Nodes { get; set; }
        public (float magnitude, float angle)[,] AdjacencyMatrix { get; set; }

        public Graph(string name, List<Node> nodes, (float, float)[,] adjacencyMatrix)
        {
            Name = name;
            Nodes = nodes;
            AdjacencyMatrix = adjacencyMatrix;

            BuildEdges(); // Create edges based on the adjacency matrix
        }

        public Graph(List<Node> nodes, (float, float)[,] adjacencyMatrix)
        {
            Name = "Graph";
            Nodes = nodes;
            AdjacencyMatrix = adjacencyMatrix;

            BuildEdges(); // Create edges based on the adjacency matrix
        }

        private void BuildEdges()
        {
            for (int i = 0; i < Nodes.Count; i++)
            {
                for (int j = 0; j < Nodes.Count; j++)
                {
                    if (AdjacencyMatrix[i, j].magnitude > 0) // Check if there's a connection
                    {
                        float angle = AdjacencyMatrix[i, j].angle;
                        float x = Nodes[j].X; // Target node x
                        float y = Nodes[j].Y; // Target node y
                        Nodes[i].Edges.Add(new Edge(Nodes[j], AdjacencyMatrix[i, j].magnitude));
                    }
                }
            }
        }

    }
}
