using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using System.Windows.Input;
using Microsoft.Extensions.Logging;
using Microsoft.Maui.Controls;
using Robobin.Models;
using RobobinApp.Views;
using Shiny.BluetoothLE;

namespace RobobinApp.ViewModels

{
    //https://docs.ros.org/en/jazzy/Concepts/Intermediate/About-Domain-ID.html <- Should just work once wifi connection
    public class MainPageViewModel : INotifyPropertyChanged
    {
        private bool _isBusy;
        private string _statusMessage;

        public ICommand GoToHelpPage { get; }




        public ICommand ConnectToRobobinCommand { get; }

        private Graph _selectedGraph;
        public ObservableCollection<Graph> Graphs { get; }
        public ICommand SelectGraphCommand { get; }
        public Graph Graph
        {
            get => _selectedGraph;
            set
            {
                _selectedGraph = value;
                OnPropertyChanged();
            }
        }
        public ObservableCollection<string> GraphNames { get; }

        public MainPageViewModel()
        {
            GoToHelpPage = new Command(OnHelpPageAccess);
            ConnectToRobobinCommand = new Command(async () => await OnConnectToRobobin());
            SelectGraphCommand = new Command<int>(OnSelectGraph);
            var graphNodes1 = new List<Node>

{
    new Node("A", 50, 50),
    new Node("B", 150, 50),
    new Node("C", 250, 50),
    new Node("Home", 350, 50),
    new Node("E", 50, 150),
    new Node("F", 150, 150),
    new Node("G", 250, 150),
    new Node("H", 350, 150),
    new Node("I", 50, 250),
    new Node("J", 150, 250),
    new Node("K", 250, 250),
    new Node("L", 350, 250),
    new Node("M", 50, 350),
    new Node("N", 150, 350),
    new Node("O", 350, 350)
};

            var graphMatrix1 = CalculatePolarConnections(graphNodes1, new bool[,]
            {
    { false, true, false, false, true, false, false, false, false, false, false, false, false, false, false },
    { true, false, true, false, false, true, false, false, false, false, false, false, false, false, false },
    { false, true, false, true, false, false, true, false, false, false, false, false, false, false, false },
    { false, false, true, false, false, false, false, true, false, false, false, false, false, false, true },
    { true, false, false, false, false, true, false, false, true, false, false, false, false, false, false },
    { false, true, false, false, true, false, true, false, false, true, false, false, false, false, false },
    { false, false, true, false, false, true, false, true, false, false, true, false, false, false, false },
    { false, false, false, true, false, false, true, false, false, false, false, true, false, false, false },
    { false, false, false, false, true, false, false, false, false, true, false, false, true, false, false },
    { false, false, false, false, false, true, false, false, true, false, true, false, false, true, false },
    { false, false, false, false, false, false, true, false, false, true, false, true, false, false, false },
    { false, false, false, false, false, false, false, true, false, false, true, false, false, false, true },
    { false, false, false, false, false, false, false, false, true, false, false, false, false, true, false },
    { false, false, false, false, false, false, false, false, false, true, false, false, true, false, true },
    { false, false, false, true, false, false, false, false, false, false, false, true, false, true, false }
            });

            var graph1 = new Graph("Lab 1", graphNodes1, graphMatrix1);

        




            var graph2Nodes = new List<Node>
            {
                new Node("Home", 50, 50),
                new Node("B", 250, 50),
                new Node("C", 150, 150),
                new Node("D", 150, 300)
            };
            var graph2Matrix = CalculatePolarConnections(graph2Nodes, new bool[,]
            {
                { false, true, true, false },
                { true, false, true, true },
                { true, true, false, true },
                { false, true, true, false }
            });
            var graph2 = new Graph("Lab 2",graph2Nodes, graph2Matrix);

            var graph3Nodes = new List<Node>
            {
                new Node("X", 75, 75),
                new Node("Home", 300, 75),
                new Node("Z", 200, 250)
            };
            var graph3Matrix = CalculatePolarConnections(graph3Nodes, new bool[,]
            {
                { false, true, true },
                { true, false, true },
                { true, true, false }
            });
            var graph3 = new Graph("Lab3", graph3Nodes, graph3Matrix);



            // Add graphs to collection
            Graphs = new ObservableCollection<Graph> { graph1, graph2, graph3 };
            GraphNames = new ObservableCollection<string>(Graphs.Select(g => g.Name));
            Graph = Graphs.First();
        }

        private void OnSelectGraph(int graphIndex)
        {
            if (graphIndex >= 0 && graphIndex < Graphs.Count)
            {
                Graph = Graphs[graphIndex];
            }
        }
        public static (float magnitude, float angle)[,] CalculatePolarConnections(List<Node> nodes, bool[,] connections)
        {
            int nodeCount = nodes.Count;
            var polarMatrix = new (float magnitude, float angle)[nodeCount, nodeCount];

            for (int i = 0; i < nodeCount; i++)
            {
                for (int j = 0; j < nodeCount; j++)
                {
                    if (i == j || !connections[i, j])
                    {
                        polarMatrix[i, j] = (0, 0);
                        continue;
                    }

        
                    float dx = nodes[j].X - nodes[i].X;
                    float dy = nodes[j].Y - nodes[i].Y;

     
                    float magnitude = (float)Math.Sqrt(dx * dx + dy * dy);

                    float angle = (float)(Math.Atan2(dy, dx) * (180 / Math.PI));

                    polarMatrix[i, j] = (magnitude, angle);
                }
            }

            return polarMatrix;
        }
        private async void OnHelpPageAccess()
        {
            Debug.WriteLine("Navigating to help page.");


            var helpPageUrl = "https://butternut-pint-1dd.notion.site/RoboBin-Help-Page-1367bb78dcfd80a49052e41ba17511f1?pvs=74";

            await Launcher.OpenAsync(new Uri(helpPageUrl));
        }
        public bool IsBusy
        {
            get => _isBusy;
            set
            {
                _isBusy = value;
                OnPropertyChanged();
            }
        }

        public string StatusMessage
        {
            get => _statusMessage;
            set
            {
                _statusMessage = value;
                OnPropertyChanged();
            }
        }

        private async Task OnConnectToRobobin()
        {
            try
            {
                IsBusy = true;


				var connectionPage = new ConnectionPage
				{
					BindingContext = new ConnectionPageViewModel()
				};
				await Application.Current.MainPage.Navigation.PushAsync(connectionPage);
			}
            catch (Exception ex)
            {
                StatusMessage = $"Failed to connect: {ex.Message}";
                await Application.Current.MainPage.DisplayAlert("Error", StatusMessage, "OK");
            }
            finally
            {
                IsBusy = false;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
