using Microsoft.Maui.Controls;
using RobobinApp.ViewModels;
using RobobinApp.Models;
using System.Diagnostics;
using RobobinApp.Views;
using Robobin.Models;

namespace RobobinApp.Views
{
    public partial class MainPage_Android : ContentPage
    {
        public MainPage_Android()
        {
            InitializeComponent();


            var drawable = (GraphicsDrawable)Resources["drawable"];
            if (BindingContext is MainPageViewModel viewModel)
            {
                drawable.SetGraph(viewModel.Graph);
            }
        }

        private void OnGraphSelected(object sender, EventArgs e)
        {

            if (sender is Picker picker && BindingContext is MainPageViewModel viewModel)
            {

                viewModel.SelectGraphCommand.Execute(picker.SelectedIndex);


                var drawable = (GraphicsDrawable)Resources["drawable"];
                var newGraph = viewModel.Graphs.FirstOrDefault(g => g.Name == viewModel.Graph.Name);


                if (newGraph != null)
                {
                    drawable.SetGraph(newGraph);
                    GraphicsView.Invalidate();
                    Debug.WriteLine("New Graph picked and drawn");
                }
                else
                {
                    Debug.WriteLine("Graph not found!");
                }
            }
        }

        private void OnGraphicsViewTapped(object sender, TappedEventArgs e)
        {
            var tapPoint = e.GetPosition(GraphicsView);
            var drawable = (GraphicsDrawable)Resources["drawable"];

            float scale = drawable.CurrentScale;
            var offset = drawable.CurrentOffset;

            var tapPointX = (float)tapPoint.Value.X;
            var tapPointY = (float)tapPoint.Value.Y;
            var nearestNode = drawable.FindNearestNode(tapPointX, tapPointY, scale, offset.X, offset.Y);

            // Highlight the nearest node
            drawable.HighlightNode(nearestNode);

            if (nearestNode != null)
            {
                // Find the home node
                Node homeNode = drawable.GetHomeNode();

                if (homeNode != null)
                {

                    List<Node> path = drawable.AStarPathfinding(nearestNode, homeNode);
                    drawable.Path = path;
                    GraphicsView.Invalidate();
                }

                DisplayAlert("Node Clicked", $"You clicked on node: {nearestNode.Name}", "OK");
            }
            else
            {
                DisplayAlert("No Node", "No node was clicked.", "OK");
            }

            // Invalidate to trigger a redraw and highlight the node
            GraphicsView.Invalidate();
        }

    }

}
