using System;
using Microsoft.Maui.Controls;

namespace RobobinApp.Views.Sides
{
    public partial class AdminBox : ContentView
    {
        public static readonly BindableProperty HeaderTitleProperty =
            BindableProperty.Create(nameof(HeaderTitle),
                                  typeof(string),
                                  typeof(AdminBox),
                                  defaultValue: "Admin:",
                                  propertyChanged: OnHeaderTitleChanged);

        public string HeaderTitle
        {
            get => (string)GetValue(HeaderTitleProperty);
            set => SetValue(HeaderTitleProperty, value);
        }

        public AdminBox()
        {
            InitializeComponent();

 
            App.WifiManager.OnMessageReceived += UpdateLatestMessage;
        }

        private void UpdateLatestMessage(string message)
        {

            MainThread.BeginInvokeOnMainThread(() =>
            {
                LatestMessageLabel.Text = message;
            });
        }

        private async void OnSendMessageClicked(object sender, EventArgs e)
        {
 
            string messageToSend = TcpMessageEntry.Text;

            if (!string.IsNullOrWhiteSpace(messageToSend))
            {

                await App.WifiManager.SendMessageAsync(messageToSend);


                TcpMessageEntry.Text = string.Empty;
            }
            else
            {
                await Application.Current.MainPage.DisplayAlert("Error", "Please enter a message to send.", "OK");
            }
        }

        protected static void OnHeaderTitleChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var control = (AdminBox)bindable;
            if (control.HeaderText != null)
            {
                control.HeaderText.Text = newValue?.ToString();
            }
        }

        protected override void OnParentChanged()
        {
            base.OnParentChanged();

            if (Parent == null)
            {
                App.WifiManager.OnMessageReceived -= UpdateLatestMessage;
            }
        }
    }
}
