import gatt

class ChatPeripheral(gatt.DeviceManager):
    def device_discovered(self, device):
        print(f"Discovered [{device.mac_address}] {device.alias()}")

class ChatService(gatt.Service):
    def __init__(self, manager):
        super().__init__(manager=manager, uuid="12345678-1234-5678-1234-56789abcdef0", primary=True)

class ChatCharacteristic(gatt.Characteristic):
    def __init__(self, service):
        super().__init__(
            uuid="12345678-1234-5678-1234-56789abcdef1", service=service,
            flags=gatt.Characteristic.FLAG_READ | gatt.Characteristic.FLAG_WRITE
        )
        self.message = ""

    def ReadValue(self, options):
        print("Client is reading the value!")
        return [dbus.Byte(c.encode()) for c in self.message]

    def WriteValue(self, value, options):
        self.message = "".join([str(byte) for byte in value])
        print(f"Received message: {self.message}")

manager = ChatPeripheral(adapter_name='hci0')
service = ChatService(manager)
chat_char = ChatCharacteristic(service)

manager.run()
