import asyncio, tkinter as tk, threading
from bleak import BleakScanner, BleakClient

# UUID for the characteristic to interact withs
CHARACTERISTIC_UUID = '00000002-710e-4a5b-8d75-3e5b444bc3cf'
RUNNING = True

class App(tk.Tk):
    def __init__(self, *args, **kwargs):
        width, height = kwargs.pop("width", 800), kwargs.pop("height", 500)
        super().__init__(*args, **kwargs)
        self.title("BLE app")
        
        self.geometry("{}x{}+{}+{}".format(width, height, (self.winfo_screenwidth() - width) // 2, (self.winfo_screenheight() - height) // 2 - 30))
        self.protocol("WM_DELETE_WINDOW", self.close)
        self.tasks = []
        
        self.rowconfigure((0, 1), weight = 1)
        self.columnconfigure(0, weight = 1)

        listboxFrame = tk.Frame(self)
        listboxFrame.grid(row = 0, column = 0, sticky = "nsew")

        self.titles = ["Devices", "Services", "Characteristics"]
        self.listboxes = []
        listboxFrame.rowconfigure(1, weight = 1, uniform = "row")
        listboxFrame.columnconfigure((0, 2, 4), weight = 1, uniform = "column")
        for i in range(3):
            tk.Label(listboxFrame, text = self.titles[i]).grid(row = 0, column = i * 2, columnspan = 2, sticky = "nsew")
            listboxFrame.columnconfigure(i * 2, weight = 1)
            lb = tk.Listbox(listboxFrame)
            lb.grid(row = 1, column = i * 2, sticky = "nsew", rowspan = 3 if i else 1)
            sb = tk.Scrollbar(listboxFrame) #scrollbar
            sb.grid(row = 1, column = (i * 2) + 1, sticky = "nsew", rowspan = 3 if i else 1)
            lb.config(yscrollcommand = sb.set)
            sb.config(command = lb.yview)
            self.listboxes.append(lb)

        self.listboxes[0].bind('<<ListboxSelect>>', self.deviceSelect)
        
        self.scanning = False
        self.devices = []
        self.scanButton = tk.Button(listboxFrame, text = "Start scanning", command = self.pressedScanButton)
        self.scanButton.grid(row = 2, column = 0, columnspan = 2, sticky = "nsew")
        self.connectButton = tk.Button(listboxFrame, text = "connect", state = "disabled")
        self.connectButton.grid(row = 3, column = 0, columnspan = 2, sticky = "nsew")

        self.bottomFrame = tk.Frame(self, bg = "navy")
        self.bottomFrame.grid(row = 1, column = 0, sticky = "nsew")

        tk.Button(self.bottomFrame, text = "Useless button").grid(row = 0, column = 0, sticky = "nsew")

    def updateDevices(self, devices):
        if self.scanning:
            self.devices = devices
            self.listboxes[0].delete(0, "end")
            names = list(map(lambda device: device.name, devices))
            # print(names)
            self.listboxes[0].insert(0, *names)
    
    def deviceSelect(self, event):
        selection = event.widget.curselection()
        if selection:
            index = selection[0]
            name = event.widget.get(index)
            print(name)
            self.connectButton.config(state = "normal")

    def pressedScanButton(self):
        if not self.scanning:
            self.scanning = True
            self.scanButton.config(text = "Stop scanning")
        else:
            self.scanning = False
            self.scanButton.config(text = "Start scanning")

    def close(self):
        global RUNNING
        RUNNING = False
        self.destroy()

root = App()

async def scanDevices(root):
        print("Scanning for BLE devices...")
        devices = await BleakScanner.discover()
        devices = list(filter(lambda device: device.name != None, devices))
        root.updateDevices(devices)
    
async def connectToDevice(device):
    print("Address: ", device.address)
    async with BleakClient(device.address) as client:
        services = client.services

        temp = services.get_characteristic(CHARACTERISTIC_UUID)
        print(temp.properties)
        print(temp.description)

async def asyncMain():
    scanningTask = None
    while RUNNING:
        if root.scanning:
            if scanningTask == None:
                scanningTask = asyncio.create_task(scanDevices(root))
                await scanningTask
                print("Created task")
            else:
                if scanningTask.done():
                    scanningTask = None
        else:
            if scanningTask != None:
                print("Cancel")
                scanningTask.cancel()
                scanningTask = None
    else:
        if scanningTask != None:
            scanningTask.cancel()


# async def main():
#     # Discover BLE devices
#     # device = await discover_devices()

#     # If a device is selected, connect and interact with it
#     # if device:
#         # await connect_and_interact(device)
#         await connect_and_interact(None)

# i = 0

if __name__ == "__main__":
    thread = threading.Thread(target = lambda: asyncio.run(asyncMain()))
    thread.start()
    root.mainloop()
    thread.join()