import time
from picographics import PicoGraphics, DISPLAY_PICO_DISPLAY
from machine import Pin
from pimoroni import Button

pico_display = PicoGraphics(display=DISPLAY_PICO_DISPLAY, rotate=180)
width, height = pico_display.get_bounds()

background_color = pico_display.create_pen(0, 0, 0)
text_color = pico_display.create_pen(255, 255, 255)

text_options = [
    ("Home", (0, 0)),
    ("Station 1", (10, 10)),
    ("Station 2", (20, 20)),
    ("Station 3", (30, 30))
]

current_index = 0
scroll_x = width
pico_display.set_font("sans")

button_pins = [12, 13, 14, 15]
buttons = [Button(pin) for pin in button_pins]

while True:
    for index, button in enumerate(buttons):
        if button.read() == True:
            if index == 0:
                current_index = (current_index + 1) % len(text_options)
                print(f"Cycled to: {text_options[current_index][0]}")
            elif index == 2:
                current_index = (current_index - 1) % len(text_options)
                print(f"Cycled to: {text_options[current_index][0]}")
            elif index == 1:
                selected_text, position = text_options[current_index]
                print(f"Sending to {selected_text}, {position}")

    text, position = text_options[current_index]

    max_width = width - 20
    scale = 2

    while True:
        text_width = pico_display.measure_text(text, scale)
        if text_width <= max_width:
            break
        scale -= 1
        if scale < 1:
            scale = 1
            break

    pico_display.set_pen(background_color)
    pico_display.clear()
    pico_display.set_pen(text_color)

    text_width = pico_display.measure_text(text, scale)
    text_x = (width - text_width) // 2
    text_y = (height // 2) - (scale * 4)

    pico_display.text(text, text_x, text_y, width, scale)
    pico_display.update()

    scroll_x -= 2
    if scroll_x < -text_width:
        scroll_x = width

    time.sleep(0.05)
