# robobin/api_node.py
import rclpy
from rclpy.node import Node
from .message_handler import MessageHandler
from .connection_manager import ConnectionManager

class ApiNode(Node):
    def __init__(self):
        super().__init__('api_node')
        self.get_logger().info("ApiNode has been started.")

        # Initialize handlers
        self.message_handler = MessageHandler(self)
        self.connection_manager = ConnectionManager(self)

        # Start connection manager
        self.connection_manager.start()

    def handle_client_connection(self, client_socket):
        """Handles incoming TCP client connections."""
        try:
            while True:
                data = client_socket.recv(1024).decode()
                if not data:
                    break
                command, *args = data.split(" ", 1)
                self.message_handler.handle_message(client_socket, command, args[0] if args else None)
        finally:
            client_socket.close()
            self.get_logger().info("Client disconnected.")

    def shutdown(self):
        """Stops the connection manager."""
        self.connection_manager.stop()

def main(args=None):
    rclpy.init(args=args)
    node = ApiNode()
    try:
        rclpy.spin(node)
    except KeyboardInterrupt:
        node.shutdown()
    finally:
        node.destroy_node()
        rclpy.shutdown()

if __name__ == '__main__':
    main()
