# robobin/message_handler.py

class MessageHandler:
    def __init__(self, api_node):
        self.api_node = api_node
        self.handlers = {
            "PING": self.handle_ping,
            "TIME": self.handle_time_request,
            "MANUALCTRL": self.handle_manual_control
        }

    def handle_message(self, client_socket, command, data):
        """Routes the command to the appropriate handler."""
        handler = self.handlers.get(command, self.handle_unknown_message)
        handler(client_socket, data)

    def handle_ping(self, client_socket, _):
        """Responds with a PONG message."""
        client_socket.sendall(b"PONG")

    def handle_time_request(self, client_socket, _):
        """Sends the current server time."""
        client_socket.sendall(time.ctime().encode())

    def handle_manual_control(self, client_socket, message):
        """Handles manual control commands: W, A, S, D."""
        directions = {
            "W": (1, 0, 0),    # Move forward
            "A": (0, 0, 1),    # Turn left
            "S": (-1, 0, 0),   # Move backward
            "D": (0, 0, -1)    # Turn right
        }
        response_data = directions.get(message.strip().upper(), (0, 0, 0))
        response = f"Manual control command received: {response_data}".encode()
        client_socket.sendall(response)
        print("Processed manual control command:", response_data)

    def handle_unknown_message(self, client_socket, _):
        """Handles unknown commands."""
        client_socket.sendall(b"Unknown command")
